﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Hadoop.Client;
using System.IO;
using System.Threading;
using System.Security.Cryptography.X509Certificates;
using Microsoft.WindowsAzure.Management.HDInsight;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            ExecuteHiveQueryWithBasicCredential();
            ExecuteHiveQueryWithCertificate();
        }

        private static void ExecuteHiveQueryWithBasicCredential()
        {
            var jobCredential = new BasicAuthCredential();
            jobCredential.UserName = "<Cluster Administration – User Name>";
            jobCredential.Password = "<Cluster Administration – Password>";
            jobCredential.Server = new Uri("https://<Cluster Name>.azurehdinsight.net//");


            // Connect to the HDInsight cluster
            var jobClient = JobSubmissionClientFactory.Connect(jobCredential);
            
            // define the Hive job
            var myHiveJobDefinition = new HiveJobCreateParameters()
            {
                JobName = "creating a Hive table",
                Query = "CREATE EXTERNAL TABLE log4jLogs ( col1 string, col2 string, col3 string, col4 string, col5 string, col6 string, col7 string ) ROW FORMAT DELIMITED  FIELDS TERMINATED BY ' ' STORED AS TEXTFILE LOCATION 'wasb:///example/data/';",
                StatusFolder = "/samplequeryoutput"
            };
            var jobResults = jobClient.CreateHiveJob(myHiveJobDefinition);
            
            // Wait for the job to complete
            WaitForHadoopJobCompletion(jobResults, jobClient);
        }
        private static void ExecuteHiveQueryWithCertificate()
        {
            // Set the variables
            string subscriptionID = "<Unique Subscription ID>";
            string clusterName = "<Cluster Name>";
            string certFriendlyName = "<Certificate Friendly Name>"; 


            // Get the certificate object from certificate store using the friendly name to identify it
            X509Store store = new X509Store();
            store.Open(OpenFlags.ReadOnly);
            X509Certificate2 certAzure = store.Certificates.Cast<X509Certificate2>().First(item => item.FriendlyName == certFriendlyName);

            if (certAzure == null)
                Console.WriteLine("No Certificate found with specified friendly name...");

            JobSubmissionCertificateCredential jobCredential = new JobSubmissionCertificateCredential(new Guid(subscriptionID), certAzure, clusterName);

            // define the Hive job
            HiveJobCreateParameters myHiveJobDefinition = new HiveJobCreateParameters()
            {
                JobName = "querying from sample table",
                StatusFolder = "/samplequeryoutput",
                Query = "SELECT country, state, count(*) AS records FROM hivesampletable GROUP BY country, state ORDER BY records DESC limit 10;"
            };

            // Submit the Hive job
            var jobClient = JobSubmissionClientFactory.Connect(jobCredential);
            JobCreationResults jobResults = jobClient.CreateHiveJob(myHiveJobDefinition);

            // Wait for the job to complete
            WaitForHadoopJobCompletion(jobResults, jobClient);

            // Print the Hive job output
            using (var responseStream = jobClient.GetJobOutput(jobResults.JobId))
            {
                StreamReader reader = new StreamReader(responseStream);
                Console.WriteLine(reader.ReadToEnd());

                Console.WriteLine("Press ENTER to continue.");
                Console.ReadLine();
            }

        }
        private static void WaitForHadoopJobCompletion(JobCreationResults jobInformation, IJobSubmissionClient clientReference)
        {
            var runningJob = clientReference.GetJob(jobInformation.JobId);
            //Keep on waiting until either the job completes or fails 
            while (runningJob.StatusCode != JobStatusCode.Completed && runningJob.StatusCode != JobStatusCode.Failed)
            {
                runningJob = clientReference.GetJob(runningJob.JobId);
                Thread.Sleep(TimeSpan.FromSeconds(10));
            }
        }

    }
}
