using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.SCP;
using Microsoft.SCP.Topology;

namespace LicensePlateReaderApplication
{
    [Active(true)]
    class Program : TopologyDescriptor
    {
        static void Main(string[] args)
        {
        }

        public ITopologyBuilder GetTopologyBuilder()
        {
            // Create LicensePlateReaderApplication storm topology
            TopologyBuilder topologyBuilder = new TopologyBuilder("LicensePlateReaderApplication");

            // Add 'vehicles' lpr spout to the topology
            // The spout emits vehicle tuples
            topologyBuilder.SetSpout(
                "vehicles",
                LPRSpout.Get,
                new Dictionary<string, List<string>>()
                {
                    {Constants.DEFAULT_STREAM_ID, new List<string>(){"vehicle"}}
                },
                1);

            // Add speed filter bolt to the topology
            // The bolt emits over-speeding vehicle tuples 
            // Shuffle grouping is used to route incoming vehicle tuple 
            //      to randomly chosen bolt
            topologyBuilder.SetBolt(
                "speedfilter",
                FilterBolt.Get,
                new Dictionary<string, List<string>>()
                {
                {Constants.DEFAULT_STREAM_ID, new List<string>(){"overspeedingvehicle"}}
                },
                1).shuffleGrouping("vehicles");

            // Add vehicle counter bolt to the topology
            // The bolt pushes the counts to a SQL Azure table
            // FieldsGrouping is used to route incoming tuples, to ensure that 
            //      tuples with same location fields are directed to same bolt instance
            topologyBuilder.SetBolt(
                "counter",
                CounterBolt.Get,
                new Dictionary<string, List<string>>()
                {
                    {Constants.DEFAULT_STREAM_ID, new List<string>(){"location", "count"}}
                }
                , 1).fieldsGrouping("speedfilter", new List<int>() { 0 });

            // Add topology configuration
            topologyBuilder.SetTopologyConfig(new Dictionary<string, string>()
                {
                    {"topology.kryo.register","[\"[B\"]"}
                });

            return topologyBuilder;
        }
    }
}

